#include "displayoptions.h"
#include <kdebug.h>
#include <qregexp.h>
#include <kcmdlineargs.h>
#include <kconfig.h>

namespace {
	const double allowedMagnifications[] = {
		0.125,
		0.25,
		0.3333,
		0.5,
		0.6667,
		0.75,
		1,
		1.25,
		1.50,
		2,
		3,
		4,
		6,
		8
	};
	const size_t numberOfMagnifications = ( sizeof( allowedMagnifications ) / sizeof( allowedMagnifications[ 0 ] ) );
}

DisplayOptions DisplayOptions::parse( KCmdLineArgs* args )
{
	DisplayOptions res;
#define CHECK_ORIENTATION( tag, value ) \
	if ( args->isSet( tag ) ) res._overrideOrientation = value; \
	if ( args->getOption( "orientation" ) == tag ) res._overrideOrientation = value;

	CHECK_ORIENTATION( "landscape", CDSC_LANDSCAPE );
	CHECK_ORIENTATION( "seascape", CDSC_SEASCAPE );
	CHECK_ORIENTATION( "portrait", CDSC_PORTRAIT);
	CHECK_ORIENTATION( "upsidedown", CDSC_UPSIDEDOWN);

	res.setMagnification( args->getOption( "scale" ).toFloat() );
	res.setPage( args->getOption( "page" ).toInt() - 1 ); // transform from 1-based into 0-based
	//res._overridePageMedia = args->getOption( "paper" );
	kdDebug(4500 ) << "Parsed options: " << res << endl;
	return res;
}

namespace {
	const char* const rformat = ".page: (\\d+); .magnification: ([\\d\\.]+); .orientation = (\\d+); .media = ([^;]*);";
	const char* const qformat = ".page: %1; .magnification: %2; .orientation = %3; .media = %4;";
}

QString DisplayOptions::toString( const DisplayOptions& options )
{
	return QString( qformat )
		.arg( options.page() )
		.arg( options.magnification() )
		.arg( options.overrideOrientation() )
		.arg( options.overridePageMedia().utf8() );
}

bool DisplayOptions::fromString( DisplayOptions& out, const QString& in )
{
	QRegExp regex( QString::fromLatin1( rformat ) );
	if ( regex.search( in ) < 0 ) return false;
	
	out.reset();
	out.setPage( regex.cap( 1 ).toInt() );
	out.setMagnification( regex.cap( 2 ).toDouble() );
	out.setOverrideOrientation( static_cast<CDSC_ORIENTATION_ENUM_e>( regex.cap( 3 ).toInt() ) );
	if ( !regex.cap( 4 ).isEmpty() ) out.setOverridePageMedia( regex.cap( 4 ) );
	return true;
}


bool DisplayOptions::canZoomIn() const
{
	return unsigned( _magnificationIndex ) < ( numberOfMagnifications - 1 );
}

bool DisplayOptions::zoomIn()
{
	if ( !canZoomIn() ) return false;
	++_magnificationIndex;
	return true;
}

bool DisplayOptions::canZoomOut() const
{
	return _magnificationIndex > 0;
}

bool DisplayOptions::zoomOut()
{
	if ( !canZoomOut() ) return false;
	--_magnificationIndex;
	return true;
}

double DisplayOptions::magnification() const
{
	return allowedMagnifications[ _magnificationIndex ];
}


void DisplayOptions::setMagnification( double newZoom ) {
	kdDebug(4500) << "DisplayOptions::setMagnification(" << newZoom << ")" << endl;
	_magnificationIndex = 0;
	while ( unsigned( _magnificationIndex ) < numberOfMagnifications 
			&& allowedMagnifications[ _magnificationIndex ] < newZoom ) ++_magnificationIndex;
	if ( unsigned( _magnificationIndex ) >= ( numberOfMagnifications - 1 ) ) {
		_magnificationIndex = numberOfMagnifications - 1;
		return;
	}
	if ( _magnificationIndex == 0 ) return;

	if ( ( allowedMagnifications[ _magnificationIndex ] - newZoom ) > ( newZoom - allowedMagnifications[ _magnificationIndex - 1 ] ) ) {
		--_magnificationIndex;
	}
	kdDebug(4500) << "DisplayOptions::setMagnification(" << newZoom  << "): nearest allowed magnification: " << magnification() << endl;
}

